require("orzia_map")
require("orzia_tile")
require("orzia_room")

require("orzia_entity")
require("orzia_player") -- extends orzia_entity
require("orzia_monster") -- extends orzia_entity

require("print_r") -- debugging library

--[[***************
****GAME*CONFIG****
***************]]--
    
--[[ View Size Configuration 

Determines number of tiles to display

x - number of tiles to display in a row
y - number of tiles to display in a column
tile_size - size of tiles in pixels

]]--

CONFIG_VIEW = {
	["size_x"] = 20, 
	["size_y"] = 20, 
	["tile_size"] = 32
}


--[[ Map Configurations

Options for map generation - probably going to add a few down the road

size_x - map width in tles
size_y - map height in tiles
rooms_max - maximum number of rooms to generate
rooms_size_min - minimum dimension for a room
rooms_size_max - maximum dimension for a room
padding - space between edge of map and possible room

]]--

CONFIG_MAP_DEFAULT = {
    ["size_x"] = 100,
    ["size_y"] = 100,
    ["rooms_max"] = 18,
    ["rooms_size_min"] = 6,
    ["rooms_size_max"] = 16
}

--[[ Animation Configuration

A bunch of shit

speed - number of miliseconds to pause between renders
step_distance - number of pixels for entity to move per frame while moving

]]--

CONFIG_ANIMATION = {
    ["speed"] = 20,
    ["step_distance"] = CONFIG_VIEW["tile_size"] / 16
}

--[[ Image Paths

Paths for images to load

tileset - tileset being used (this is changing soon)
titleimage - the image to display at game launch

]]--

CONFIG_PATHS = {

    ["titleimage"] = "/images/title.png",

    ["tileset_entities"] = "/images/tileset.png",
    ["tileset_map_default"] = "/images/map_test.png"

}

--[[ Tile Configurations

work in progress

]]--

CONFIG_TILES = {

    -- every tileset should have a blank tile in the 0,0 position
    ["empty"] = {x = 0, y = 0},

    -- map tiles
    ["map_floor"] = {x = 1, y = 0},
    ["map_wall"] = {x = 2, y = 0},
    ["map_corner"] = {x = 3, y = 0},
    ["map_corridor"] = {x = 4, y = 0},
    ["map_island"] = {x = 5, y = 0}, 
    ["map_nub"] = {x = 6, y = 0},


    -- entity tiles
    ["entity_player"] = {x = 18, y = 20},
    ["entity_m_snake"] = {x = 13, y = 24}

}

--[[********** 
**END CONFIG**
**********]]--

function love.load() -- game initialization

	-- Initialize game state
	GAME_STATE = "main_menu"

	-- Generates seed for random number generator
	math.randomseed(os.time())

end


function love.update() -- game logic update

	if GAME_STATE == "new_dungeon" then

		-- initialize map
		map = orzia_map.create(CONFIG_MAP_DEFAULT, initializeTileset(CONFIG_PATHS["tileset_map_default"], CONFIG_VIEW))
		map:generateMap(CONFIG_VIEW)
		map:determineTileGraphics(CONFIG_TILES)

		-- initialize player
		player = orzia_player.create(map.rooms[1].centerx, map.rooms[1].centery, CONFIG_TILES["entity_player"], true, "player")

		-- initialize test monster
		monster = orzia_monster.create(map.rooms[2].centerx, map.rooms[2].centery, CONFIG_TILES["entity_m_snake"], true, "snake")

		-- initialize entities
		entities = {}

		table.insert(entities, player)
		table.insert(entities, monster)

		-- initialize entity spritebatch
		entity_tileset = initializeTileset(CONFIG_PATHS["tileset_entities"], CONFIG_VIEW)
		entity_spritebatch = love.graphics.newSpriteBatch(entity_tileset.image, map.size_x * map.size_y)

		GAME_STATE = "dungeon"

	end

	if GAME_STATE == "dungeon" then

		-- movement animation

		for i, entity in ipairs(entities) do

			if entity.moving == true then

				ANIMATION = entity:continueMoving(CONFIG_ANIMATION["step_distance"], CONFIG_VIEW["tile_size"])

			end

		end

	end

end


function love.draw() -- render
	
	if GAME_STATE == "dungeon" then

		-- determine offset
		offsetX = (player.x  - CONFIG_VIEW["size_x"] / 2) * CONFIG_VIEW["tile_size"]
		offsetY = (player.y + 2 - CONFIG_VIEW["size_y"] / 2) * CONFIG_VIEW["tile_size"]

		-- draw map
		map:draw(CONFIG_VIEW, offsetX, offsetY, player)

		-- draw entities
		entity_spritebatch:clear()

		for i, entity in ipairs(entities) do
			if entity.name ~= "player" then
				entity_spritebatch:addq(entity_tileset[entity.graphic.x][entity.graphic.y], 
					entity.x * CONFIG_VIEW["tile_size"] + entity.movestep * entity.direction["x"], 
					entity.y * CONFIG_VIEW["tile_size"] + entity.movestep * entity.direction["y"])
			end
		end

		love.graphics.draw(entity_spritebatch, -offsetX - player.movestep * player.direction["x"], -offsetY - player.movestep * player.direction["y"])
		--end entities

		-- draw player
		entity_spritebatch:clear()

		entity_spritebatch:addq(entity_tileset[player.graphic.x][player.graphic.y], 
			player.x * CONFIG_VIEW["tile_size"], 
			player.y * CONFIG_VIEW["tile_size"])

		love.graphics.draw(entity_spritebatch, -offsetX, -offsetY)

	elseif GAME_STATE == "main_menu" then

		ANIMATION = false

		love.graphics.draw(love.graphics.newImage(CONFIG_PATHS["titleimage"]))

	end

	love.timer.sleep(CONFIG_ANIMATION["speed"])
	
end


-- mouse handling

function love.mousepressed(x, y, button)
	
	--[[

	title screen controls

	]]--

	if GAME_STATE == "main_menu" then

		if button == "l" then
			if (x > 90 and x < 225 and 
				y > 325 and y < 375) then
				GAME_STATE = "new_dungeon"
			end
		end

	end

	if GAME_STATE == "dungeon" then

		if button == "l" then

			pr(map.tiles[math.ceil((x + offsetX) / CONFIG_VIEW["tile_size"]) - 1][math.ceil((y + offsetY) / CONFIG_VIEW["tile_size"]) - 1])

		end

	end

end


function love.mousereleased(x, y, button)

end

-- end mouse handling


-- keyboard handling

function love.keypressed(key, unicode)

	pr(ANIMATION)

	if GAME_STATE == "dungeon" and ANIMATION == false then

		if key == "up" then
			player:move(0, -1)
		elseif key == "down" then
			player:move(0, 1)
		elseif key == "right" then
			player:move(1, 0)
		elseif key == "left" then
			player:move(-1,0)
		end

		if key == "h" then
			monster:move(math.random(-1,1), math.random(-1,1))
		end

	end

end


function love.keyreleased(key, unicode)

end

-- end keyboard handling


-- returns a spritebatch to use for a given tileset
function initializeTileset(path, view_config)

	local tileset = {}
	local tilesetImage = love.graphics.newImage(path)

	local tile_size = view_config["tile_size"]
	local view_size_x = view_config["size_x"]
	local view_size_y = view_config["size_y"]

	for x = 0, tilesetImage:getWidth() / tile_size do
		tileset[x] = {}
		for y = 0, tilesetImage:getHeight() / tile_size do
			tileset[x][y] = love.graphics.newQuad(x * tile_size, y * tile_size, tile_size, tile_size, tilesetImage:getWidth(), tilesetImage:getHeight())
		end
	end

	tileset.image = tilesetImage

	return tileset

end