orzia_map = {}
orzia_map.__index = orzia_map

function orzia_map.create(map_config, tileset)

	local map = {}
	setmetatable(map, orzia_map)

	-- see main.lua for explanation of variables
	map.size_x = map_config["size_x"]
	map.size_y = map_config["size_y"]
	map.rooms_max = map_config["rooms_max"]
	map.rooms_size_min = map_config["rooms_size_min"]
	map.rooms_size_max = map_config["rooms_size_max"]

	map.tileset = tileset
	map.spritebatch = love.graphics.newSpriteBatch(map.tileset.image, map.size_x * map.size_y)

	return map

end

function orzia_map:generateMap(view_config)

	-- create empty table for tiles
	self.tiles = {}

	for x = 0, self.size_x - 1 do
		self.tiles[x] = {}
		for y = 0, self.size_y - 1 do
			self.tiles[x][y] = orzia_tile.create(true, x, y, view_config["tile_size"])
		end
	end


	--generate some rooms
	self.rooms = {}

	for numRooms = 1, self.rooms_size_max do

		local intersect = true
		local room = {}

		-- create rooms until generating one that does not overlap existing rooms
		while intersect == true do

			local w = math.random(self.rooms_size_min, self.rooms_size_max)
			local h = math.random(self.rooms_size_min, self.rooms_size_max)
			local x = math.random(view_config["size_x"], self.size_x - w - view_config["size_x"])
			local y = math.random(view_config["size_y"], self.size_y - h - view_config["size_y"])

			room = orzia_room.create(x, y, w, h)

			intersect = false

			if numRooms > 1 then

				for i, other_room in ipairs(self.rooms) do

					if room:doesIntersect(other_room) == true then
						intersect = true
					end

				end

			end

		end

		self.rooms[numRooms] = room

		--unblock new room
		for x = room.x1, room.x2 do
			for y = room.y1, room.y2 do
				self.tiles[x][y].blocked = false
			end
		end

		--build corridor between previous room and this one
		if numRooms > 1 then

			if math.random() == 0 then

				self:buildHorizontalTunnel(self.rooms[numRooms - 1].centerx, room.centerx, self.rooms[numRooms - 1].centery)
				self:buildVerticalTunnel(self.rooms[numRooms - 1].centery, room.centery, room.centerx)

			else

				self:buildVerticalTunnel(self.rooms[numRooms - 1].centery, room.centery, self.rooms[numRooms - 1].centerx)
				self:buildHorizontalTunnel(self.rooms[numRooms - 1].centerx, room.centerx, room.centery)

			end

		end

	end
	-- end room generation

	-- pave edges
	for x = 0, self.size_x - 1 do
		for y = 0, self.size_y - 1 do
			if x == 0 or y == 0 or x == self.size_x - 1 or y == self.size_y - 1 then
				self.tiles[x][y].blocked = true
			end
		end
	end
	
end

function orzia_map:determineTileGraphics(tile_config)

	for x = 0, self.size_x - 1 do
		for y = 0, self.size_y - 1 do
			if self.tiles[x][y]:countWalls(self.tiles) == 9 then

				self.tiles[x][y].graphic = tile_config["empty"]

			else

				if self.tiles[x][y].blocked == true then

					local adjacentWalls = self.tiles[x][y]:countWallsAdjacent(self.tiles)

					if adjacentWalls == 0 then 
						
						self.tiles[x][y].graphic = tile_config["map_island"] 

					elseif adjacentWalls == 1 then 

						self.tiles[x][y].graphic = tile_config["map_nub"]

					elseif adjacentWalls == 2 then 

						self.tiles[x][y].graphic = tile_config["map_corner"] -- need to detect whether its a corner or a corridor

					elseif adjacentWalls == 3 then 

						self.tiles[x][y].graphic = tile_config["map_wall"]

							if math.random(3) == 2 then
								self.tiles[x][y].graphic.x = 0
							else
								self.tiles[x][y].graphic.x = 2
							end

					end

				else

					self.tiles[x][y].graphic = tile_config["map_floor"]

				end
			end
		end
	end

end

function orzia_map:buildHorizontalTunnel(x1, x2, y)

	for x = math.min(x1, x2), math.max(x1, x2) do
		self.tiles[x][y].blocked = false
	end

end

function orzia_map:buildVerticalTunnel(y1, y2, x)

	for y = math.min(y1, y2), math.max(y1, y2) do
		self.tiles[x][y].blocked = false
	end

end

function orzia_map:draw(view_config, offsetx, offsety, player)

	self.spritebatch:clear()

	local tile_size = view_config["tile_size"]
	local view_size_x = view_config["size_x"]
	local view_size_y = view_config["size_y"]
		
	for x = offsetx / tile_size - 1, offsetx / tile_size + view_size_x do
		for y = offsety / tile_size - 1, offsety / tile_size + view_size_y do
			if self.tiles[x] ~= nil and self.tiles[x][y] ~= nil then
				self.spritebatch:addq(self.tileset[self.tiles[x][y].graphic.x][self.tiles[x][y].graphic.y],
					x * tile_size,
					y * tile_size,
					self.tiles[x][y].graphic.rotation,
					1,
					1,
					self.tiles[x][y].graphic.ox,
					self.tiles[x][y].graphic.oy)
			end
		end
	end

	love.graphics.draw(self.spritebatch, -offsetx - player.movestep * player.direction["x"], -offsety - player.movestep * player.direction["y"])

end